use master
GO
IF EXISTS (SELECT * FROM sys.databases WHERE name = 'Sample1')
	DROP DATABASE Sample1
GO
CREATE DATABASE Sample1
GO
ALTER DATABASE Sample1
	SET ENABLE_BROKER
GO
use Sample1
GO
IF NOT EXISTS (SELECT * FROM sys.symmetric_keys WHERE name = '##MS_DatabaseMasterKey##')
	CREATE MASTER KEY ENCRYPTION BY password='pa$$word'
GO
CREATE TABLE InvoiceReporting
(InvoiceId INT PRIMARY KEY,
CustomerId INT,
LineItems INT,
ItemQty INT,
InvoiceDate DATETIME,
TotalDiscounts MONEY,
InvoiceTotal MONEY,
ShippingAddress NVARCHAR(100),
ShippingCity NVARCHAR(50),
ShippingState NVARCHAR(2),
ShippingZip NVARCHAR(20),
ShippingCountry NVARCHAR(2),
BillingAddress NVARCHAR(100),
BillingCity NVARCHAR(50),
BillingState NVARCHAR(2),
BillingZip NVARCHAR(20),
BillingCountry NVARCHAR(20))
GO
CREATE TABLE CustomerReporting
(CustomerId INT,
FirstName VARCHAR(50),
LastName VARCHAR(50))
GO
CREATE TABLE AddressReporting
(AddressId INT PRIMARY KEY,
AddressTypeId INT,
CustomerID INT,
Address1 NVARCHAR(100),
City NVARCHAR(50),
State NVARCHAR(2),
Zip NVARCHAR(20),
Country NVARCHAR(2))
GO
CREATE TABLE EmployeeReporting
(EmployeeId INT PRIMARY KEY,
FirstName VARCHAR(50),
LastName VARCHAR(50),
Email VARCHAR(255),
UserName VARCHAR(255),
Active INT)
GO
CREATE MESSAGE TYPE MT_Shipping
GO
CREATE CONTRACT CT_Shipping AUTHORIZATION dbo
(MT_Shipping SENT BY ANY)
GO
CREATE QUEUE Q_Sender
GO
CREATE QUEUE Q_Receiver
GO
CREATE SERVICE S_Sender AUTHORIZATION dbo
ON QUEUE Q_Sender
(CT_Shipping)
GO
CREATE SERVICE S_Receiver AUTHORIZATION dbo
ON QUEUE Q_Receiver
(CT_Shipping)
GO
GRANT CONTROL ON SERVICE :: S_Receiver TO PUBLIC
GO
CREATE ROUTE [SampleRoute] AUTHORIZATION dbo
	WITH SERVICE_NAME = 'S_Sender',
	ADDRESS='TCP://SQL2K5-1:1234'	
GO
CREATE PROCEDURE ProcessEmployee
	@XML XML
AS
DECLARE @hdoc AS INT

exec sp_xml_preparedocument @hdoc OUTPUT, @xml

	INSERT INTO EmployeeReporting
	(EmployeeId, FirstName, LastName, Email, UserName, Active)
	SELECT EmployeeID, FirstName, LastName, Email, UserName, 1
	FROM OPENXML(@hdoc, N'/d/i')
	WITH ([EmployeeId] INT '@EmployeeId',
	[FirstName] VARCHAR(50) '@FirstName',
	[LastName] VARCHAR(50) '@LastName',
	[Email] VARCHAR(255) '@Email',
	[UserName] VARCHAR(255) '@UserName',
	[Command] CHAR(1) '../@Command') a
	WHERE Command = 'I'
		AND NOT EXISTS (
			SELECT * 
			FROM EmployeeReporting 
			WHERE a.EmployeeID = EmployeeReporting.EmployeeID
		)

	UPDATE EmployeeReporting
		SET EmployeeReporting.FirstName = a.FirstName,
			EmployeeReporting.LastName = a.LastName,
			EmployeeReporting.Email = a.Email,
			EmployeeReporting.UserName = a.UserName,
			EmployeeReporting.Active = case when Command = 'I' then 1 else 0 end
	FROM OPENXML(@hdoc, N'/d/i')
	WITH ([EmployeeId] INT '@EmployeeId',
	[FirstName] VARCHAR(50) '@FirstName',
	[LastName] VARCHAR(50) '@LastName',
	[Email] VARCHAR(255) '@Email',
	[UserName] VARCHAR(255) '@UserName',
	[Command] CHAR(1) '../@Command',
	[DeletedEmployeeID] INT '@Deleted') a
	WHERE case when a.Command = 'I' then a.EmployeeID else a.DeletedEmployeeId end = EmployeeReporting.EmployeeID

exec sp_xml_removedocument @hdoc
GO
CREATE PROCEDURE ProcessAddress
	@XML XML
AS
DECLARE @hDoc AS INT

exec sp_xml_preparedocument @hdoc OUTPUT, @xml

	INSERT INTO AddressReporting
	(AddressId, AddressTypeId, CustomerId, Address1, City, State, Zip, Country)
	SELECT AddressId, AddressTypeId, CustomerId, Address1, City, State, Zip, Country
	FROM OPENXML(@hdoc, N'/d/i')
	WITH (AddressId INT '@AddressId',
	AddressTypeId INT '@AddressTypeId',
	CustomerID INT '@CustomerId',
	Address1 NVARCHAR(100) '@Address1',
	City NVARCHAR(50) '@City',
	State NVARCHAR(2) '@State',
	Zip NVARCHAR(20) '@Zip',
	Country NVARCHAR(2) '@Country',
	[Command] CHAR(1) '../@Command') a
	WHERE Command = 'I'
		AND NOT EXISTS (
			SELECT * 
			FROM AddressReporting 
			WHERE a.AddressId = AddressReporting.AddressId
		) 

	UPDATE AddressReporting
		SET AddressReporting.AddressId = a.AddressId, 
			AddressReporting.AddressTypeId = a.AddressTypeId, 
			AddressReporting.CustomerId = a.CustomerId, 
			AddressReporting.Address1 = a.Address1, 
			AddressReporting.City = a.City, 
			AddressReporting.State = a.State, 
			AddressReporting.Zip = a.Zip, 
			AddressReporting.Country = a.Country
	FROM OPENXML(@hdoc, N'/d/i')
	WITH (AddressId INT '@AddressId',
	AddressTypeId INT '@AddressTypeId',
	CustomerID INT '@CustomerId',
	Address1 NVARCHAR(100) '@Address1',
	City NVARCHAR(50) '@City',
	State NVARCHAR(2) '@State',
	Zip NVARCHAR(20) '@Zip',
	Country NVARCHAR(2) '@Country',
	[Command] CHAR(1) '../@Command') a
	WHERE Command = 'I'
		AND NOT EXISTS (
			SELECT * 
			FROM AddressReporting 
			WHERE a.AddressId = AddressReporting.AddressId
		) 

exec sp_xml_removedocument @hdoc
GO

CREATE PROCEDURE ProcessCustomer
	@XML XML
AS
DECLARE @hdoc AS INT

exec sp_xml_preparedocument @hdoc OUTPUT, @xml

	INSERT INTO CustomerReporting
	(CustomerId, FirstName, LastName)
	SELECT CustomerId, FirstName, LastName
	FROM OPENXML(@hdoc, N'/d/i')
	WITH (CustomerId INT '@CustomerId',
	[FirstName] VARCHAR(50) '@FirstName',
	[LastName] VARCHAR(50) '@LastName',
	[Command] CHAR(1) '../@Command') a
	WHERE Command = 'I'
		AND NOT EXISTS (
			SELECT * 
			FROM CustomerReporting 
			WHERE a.CustomerID = CustomerReporting.CustomerID
		)

	UPDATE CustomerReporting
		SET CustomerReporting.FirstName = a.FirstName,
			CustomerReporting.LastName = a.LastName
	FROM OPENXML(@hdoc, N'/d/i')
	WITH ([CustomerId] INT '@CustomerId',
	[FirstName] VARCHAR(50) '@FirstName',
	[LastName] VARCHAR(50) '@LastName',
	[Command] CHAR(1) '../@Command') a
	WHERE a.CustomerId = CustomerReporting.CustomerId

exec sp_xml_removedocument @hdoc
GO
CREATE PROCEDURE ReceiveMessage AS
DECLARE @xml AS XML
DECLARE @dialog_handle as uniqueidentifier
DECLARE @hdoc AS INT
DECLARE @Table as SYSNAME
DECLARE @Direction as CHAR(1)

WHILE 1=1
BEGIN

	SET @dialog_handle = null

	WAITFOR ( RECEIVE TOP (1) @dialog_handle = conversation_handle, @xml = cast(message_body as xml)
		FROM Q_Receiver), TIMEOUT 1000

	IF @dialog_handle IS NULL
		break

	exec sp_xml_preparedocument @hdoc OUTPUT, @xml

	SELECT @Table=[Table], @Direction=[Command]
	FROM OPENXML(@hdoc, N'/d')
	WITH ([Table] sysname '@Table',
		[Command] CHAR(1) '@Command')

	IF @Table = 'Employee'
		exec ProcessEmployee @XML=@xml
	IF @Table = 'Customer'
		exec ProcessCustomer @XML=@xml
	IF @Table = 'Address'
		exec PRocessAddress @XML=@xml

	exec sp_xml_removedocument @hdoc

	END CONVERSATION @dialog_handle
END
GO