use master
GO
IF EXISTS (SELECT * FROM sys.databases WHERE name = 'Sample1')
	DROP DATABASE Sample1
GO
CREATE DATABASE Sample1
GO
ALTER DATABASE Sample1
	SET ENABLE_BROKER
GO
use Sample1
GO
IF NOT EXISTS (SELECT * FROM sys.symmetric_keys WHERE name = '##MS_DatabaseMasterKey##')
	CREATE MASTER KEY ENCRYPTION BY password='pa$$word'
GO
CREATE TABLE Customer
(CustomerId INT IDENTITY(1,1) PRIMARY KEY,
FirstName varchar(50),
LastName varchar(50))
GO
CREATE TABLE Employee
(EmployeeId INT IDENTITY (1,1) PRIMARY KEY,
FirstName VARCHAR(50),
LastName VARCHAR(50),
Email VARCHAR(255),
UserName VARCHAR(255))
GO
CREATE TABLE AddressType
(AddressTypeId INT IDENTITY(1,1) PRIMARY KEY,
AddressTypeName VARCHAR(10))
GO
CREATE TABLE Address
(AddressId INT IDENTITY(1,1) PRIMARY KEY,
AddressTypeId INT,
CustomerId INT,
Address1 NVARCHAR(100),
City NVARCHAR(50),
State NVARCHAR(2),
Zip NVARCHAR(20),
Country NVARCHAR(2))
GO
CREATE TABLE Invoice
(InvoiceId INT IDENTITY(1,1) PRIMARY KEY,
CustomerId INT,
SellingEmployeeId INT,
ShippingAddressId INT,
BillingAddressId INT,
InvoiceTotal MONEY,
InvoiceDate DATETIME)
GO
CREATE TABLE InvoiceLine
(InvoiceId INT,
LineId INT,
ProductId INT,
Qty INT,
CostPerItem MONEY,
LineDiscount MONEY,
LineTotal MONEY
CONSTRAINT PK_InvoiceLine PRIMARY KEY (InvoiceId, LineId))
GO
CREATE MESSAGE TYPE MT_Shipping
GO
CREATE CONTRACT CT_Shipping AUTHORIZATION dbo
(MT_Shipping SENT BY ANY)
GO
CREATE QUEUE Q_Sender
GO
CREATE QUEUE Q_Receiver
GO
CREATE SERVICE S_Sender AUTHORIZATION dbo
ON QUEUE Q_Sender
(CT_Shipping)
GO
CREATE SERVICE S_Receiver AUTHORIZATION dbo
ON QUEUE Q_Receiver
(CT_Shipping)
GO
/*
To make this work on your system, change the ADDRESS
Parameter below to the name of your remote machine.
*/
CREATE ROUTE [SampleRoute] AUTHORIZATION dbo
	WITH SERVICE_NAME = 'S_Receiver',
	ADDRESS='TCP://SQL2K5-2:1234'	
GO
CREATE PROCEDURE SendShippingMessage
	@Message XML
AS
DECLARE @handle UNIQUEIDENTIFIER

IF @Message IS NULL
	return (0)

BEGIN DIALOG @handle
FROM SERVICE S_Sender
TO SERVICE 'S_Receiver'
ON CONTRACT CT_Shipping
WITH ENCRYPTION=OFF;

SEND ON CONVERSATION @handle
	MESSAGE TYPE MT_Shipping
	(@Message)
GO
CREATE TRIGGER t_Invoice ON dbo.Invoice
FOR INSERT, UPDATE, DELETE
AS
DECLARE @Message XML

SET @Message = (
	SELECT case when i.InvoiceId IS NULL THEN 'D' ELSE 'I' END AS 'Command', 'Invoice' as 'Table', d.InvoiceId as 'Deleted', i.*
	FROM INSERTED i
	LEFT OUTER JOIN DELETED d ON i.InvoiceID = d.InvoiceID

	FOR XML AUTO
	)

exec SendShippingMessage @Message=@Message
GO
CREATE TRIGGER t_InvoiceLine ON dbo.InvoiceLine
FOR INSERT, UPDATE, DELETE
AS
DECLARE @Message XML

SET @Message = (
	SELECT case when i.InvoiceId IS NULL THEN 'D' ELSE 'I' END AS 'Command', 'InvoiceLine' as 'Table', d.InvoiceId as 'DeletedInvoice', isnull(d.LineId, 0) as 'DeletedLineId', i.*
	FROM INSERTED i
	LEFT OUTER JOIN DELETED d ON i.InvoiceID = d.InvoiceID
		and i.LineId = d.LineId
	FOR XML AUTO
	)

exec SendShippingMessage @Message=@Message
GO
CREATE TRIGGER t_Address ON dbo.Address
FOR INSERT, UPDATE, DELETE
AS
DECLARE @Message XML

SET @Message = (
	SELECT case when i.AddressId IS NULL THEN 'D' ELSE 'I' END AS 'Command', 'Address' as 'Table', d.AddressId as 'Deleted', i.*
	FROM INSERTED i
	LEFT OUTER JOIN DELETED d ON i.AddressID = d.AddressId
	FOR XML AUTO
	)

exec SendShippingMessage @Message=@Message
GO
CREATE TRIGGER t_Employee ON dbo.Employee
FOR INSERT, UPDATE, DELETE
AS
DECLARE @Message XML

SET @Message = (
	SELECT case when i.EmployeeId IS NULL THEN 'D' ELSE 'I' END AS 'Command', 'Employee' as 'Table', d.EmployeeId as 'Deleted', i.*
	FROM INSERTED i
	LEFT OUTER JOIN DELETED d ON i.EmployeeID = d.EmployeeId
	FOR XML AUTO
	)

exec SendShippingMessage @Message=@Message
GO
CREATE TRIGGER t_Customer ON dbo.Customer
FOR INSERT, UPDATE, DELETE
AS
DECLARE @Message XML

SET @Message = (
	SELECT case when i.CustomerId IS NULL THEN 'D' ELSE 'I' END AS 'Command', 'Customer' as 'Table', d.CustomerId as 'Deleted', i.*
	FROM INSERTED i
	LEFT OUTER JOIN DELETED d ON i.CustomerId = d.CustomerId
	FOR XML AUTO
	)

exec SendShippingMessage @Message=@Message
GO
CREATE PROCEDURE ProcessAck
AS
DECLARE @xml AS XML
DECLARE @dialog_handle as uniqueidentifier
while 1=1
begin
	set @dialog_handle = null

	WAITFOR ( RECEIVE TOP (1) @dialog_handle = conversation_handle, @xml = cast(message_body as xml)
		FROM Q_Sender), TIMEOUT 1000

	IF @dialog_handle IS NULL
		break

	END CONVERSATION @dialog_handle
end
GO
ALTER QUEUE Q_Sender
	WITH ACTIVATION (STATUS=ON, 
		PROCEDURE_NAME=ProcessAck,
		MAX_QUEUE_READERS=1,
		EXECUTE AS SELF)
GO

INSERT INTO Customer
(FirstName, LastName)
SELECT 'Bob', 'Smith'
UNION
SELECT 'Joe', 'Brown'
UNION
SELECT 'Sarah', 'Jones'
UNION
SELECT 'Bob', 'Eubanks'
UNION
SELECT 'Sarah', 'Connor'
UNION
SELECT 'William', 'Gates'
UNION
SELECT 'Martha', 'Stewart'
UNION
SELECT 'Patrick', 'Stewart'
UNION
SELECT 'Kira', 'Naris'
GO
INSERT INTO Employee
(FirstName, LastName, Email, UserName)
SELECT 'Bill', 'Jones', 'bjones@widgit.com', 'bjones'
UNION ALL
SELECT 'Kevin', 'Smith', 'ksmith@widgit.com', 'ksmith'
UNION ALL
SELECT 'Sarah', 'Parker', 'sparker@widgit.com', 'sparker'
UNION ALL
SELECT 'Ferris', 'Bueller', 'fbueller@widgit.com', 'fbueller'
UNION ALL
SELECT 'Ed', 'Rooney', 'erooney@widgit.com', 'erooney'
GO
INSERT INTO Address
(AddressTypeId, CustomerId, Address1, City, State, Zip, Country)
SELECT 1, 1, '13 Main Street', 'Moorpark', 'CA', '95487', 'US'
UNION ALL
SELECT 1, 2, '54 Hazel Ave', 'Norco', 'CA', '45211', 'US'
UNION ALL
SELECT 1, 3, '827 Smith St', 'San Diego', 'CA', '54488', 'US'
UNION ALL
SELECT 1, 4, '39897 The Place Way', 'London', '', '837 WSA', 'UK'
UNION ALL
SELECT 1, 5, '297 Park Place', 'Madrid', '', 'JDU-938', 'ES'
UNION ALL
SELECT 1, 6, '198 Some Street', 'Carsbad', 'CA', '95847', 'US'
UNION ALL
SELECT 1, 7, '234098 Anystreet', 'Anytown', 'NV', '54874', 'US'
UNION ALL
SELECT 1, 8, '32 Beach Way', 'Ventura', 'CA', '54785', 'US'
UNION ALL
SELECT 1, 9, '654 Moorpark Road', 'Thousand Oaks', 'CA', '92541', 'US'

INSERT INTO Address
(AddressTypeId, CustomerId, Address1, City, State, Zip, Country)
SELECT 2, CustomerId, Address1, City, State, Zip, Country
FROM Address
GO
INSERT INTO AddressType
(AddressTypeName)
SELECT 'Shipping'
UNION ALL
SELECT 'Billing'
GO
/*
UPDATE Employee
	SET LastName = 'Changed'
WHERE EmployeeId = 3
*/